# pyright: reportUnusedImport=false
from commands.create import create
from commands.download import download
from commands.info import info
from commands.predict import predict
from commands.test import test
from commands.train import train
from commands.compare import compare
